/* ***************************************************************************
 *
 *	This program is free software; you can redistribute it and/or modify it
 * 	under the terms of the Shared Code License (SCL) version 1.0 as published
 *	by www.IntellectualHeaven.com. You should have received a copy of the SCL
 *	1.0 license, look for file license.txt, along with this software; if not,
 *	then please write to x_pankaj_x@intellectualheaven.com for an electronic
 *	copy of the same.
 *
 * File name:
 * 		Sf.h
 * 
 * Description:
 * 		This file contains definitions and prototypes which are common for both
 *		CUI and GUI build of SnifFire
 * 	
 * Revision:
 * --------------------------------------------------------------------------
 *		2005-04-24	xpankajx@intellectualheaven.com
 *		First version
 * --------------------------------------------------------------------------
 * 	Copyright (c) : Pankaj Garg
 * **************************************************************************/
#ifndef _SF_H_
#define _SF_H_

//System-Includes!!!
#include <windows.h>
#include <vector>
#include <string>

//Local-Includes!!!
#include "rawnetif.h"

// Global adapter handle
extern HANDLE ghAdapter;

/*
 * Maximum size of a packet that can be received on ethernet
 */
#define MAX_ETHERNET_PACKET_LENGTH		1514

#define ETHERNET_MAXIMUM_DATA_LENGTH			0x05DC

#define ETHERNET_CONTAINS_IPV4_FRAME_STR		L"IPv4 Datagram"
#define ETHERNET_CONTAINS_IPV6_FRAME_STR		L"IPv6 Datagram"
#define ETHERNET_CONTAINS_ARP_FRAME_STR			L"ARP Datagram"
#define ETHERNET_CONTAINS_RARP_FRAME_STR		L"RARP Datagram"
#define ETHERNET_CONTAINS_UNKNOWN_FRAME_STR		L"Unknown"

#define ETHERNET_FORMAT_ETHERNET_II_STR			L"Ethernet_II"
#define ETHERNET_FORMAT_802_3_STR				L"802.3"


// Basic data types
typedef unsigned char	u8;
typedef unsigned short 	u16;
typedef unsigned long 	u32;

/*
 * Protocol numbers to represent the protocol contained
 * inside an IP datagram
 */
#define TCP_PROTOCOL_NUM		0x06

/*
 * Structure to store action taken on a packet
 * when firewall is invoked
 */
enum FIREWALL_ACTION
{
	ACTION_NONE,
	ACTION_ACCESS_GRANTED,
	ACTION_ACCESS_DENIED
};

/*
 * structures to store a packet data
 */

#pragma pack (push, 1)

enum Ethernet_Type
{
	ETHERNET_II,
	ETHERNET_802_3
};


/*
 * Ethernet Frame Types related information
 *		0x0800		- IP Datagram
 *		0x0806		- ARP/RARP Datagram
 *	>=	0x05DD		- other/unknown Datagram
 *	<	0x05DD		- Frame type actually contains length of data
 */
enum Datagram_Type
{
	IPV4_DATAGRAM	= 0x0800,
	IPV6_DATAGRAM	= 0x86DD,
	ARP_DATAGRAM	= 0x0806,
	RARP_DATAGRAM	= 0x8035,
	UNKNOWN			= 0x0000
};

struct Ethernet_Header
{
	unsigned char	sourceMAC[6];
	unsigned char	destMAC[6];
	unsigned short	type;
};


struct Basic_IP_Header
{
	unsigned char	verAndHLen;
	unsigned char	tos;
	unsigned short	totalLength;
	unsigned short	identification;
	unsigned short	flagsAndOffset;
	unsigned char	ttl;
	unsigned char	proto;
	unsigned short	checksum;
	union
	{
		unsigned char	sourceIPAddrOctets[4];
		long int		sourceIPAddr;
	};
	union
	{
		unsigned char	destIPAddrOctets[4];
		long int		destIPAddr;
	};
};


struct Basic_TCP_Header
{
	unsigned short	sourcePort;
	unsigned short	destPort;
	unsigned long	seqNumber;
	unsigned long	ackNumber;
	unsigned char	hLen;		// Use only high order 4 bits
	unsigned char	codeBits;	// Use only low order 6 bits
	unsigned short	window;
	unsigned short	checksum;
	unsigned short	urgentPointer;
};


struct Ethernet_II_Packet
{
	Ethernet_Header		header;
	unsigned char		ip_data[1];
};


struct Ethernet_802_3_Packet
{
	Ethernet_Header		header;
	unsigned char		dsap;
	unsigned char		ssap;
	unsigned char		ctrl;
	unsigned char		orgID[3];
	unsigned short		type;
	unsigned char		ip_data[1];
};

#pragma pack(pop)

class PacketData
{
public:
	PacketData(int inLength, void *inData)
	{
		length	= inLength;
		data	= new unsigned char[length];

		if (data)
		{
			memcpy(data, inData, inLength);
		}
		
		raw_ip_data		= NULL;
		raw_tcp_data	= NULL;
		action 			= ACTION_NONE;
	}

	~PacketData()
	{
		length = 0;
		delete [] data;
	}

	union
	{
		unsigned char					*data;
		Ethernet_Header					*E_Header;
		Ethernet_II_Packet				*E_II_Data;
		Ethernet_802_3_Packet			*E_802_3_Data;
	};
	
	union
	{
		Basic_IP_Header		*ip;
		unsigned char		*raw_ip_data;
	};
	
	union
	{
		Basic_TCP_Header	*tcp;
		unsigned char		*raw_tcp_data;
	};

	unsigned int		length;
	FIREWALL_ACTION		action;
	Ethernet_Type		ethernetType;
	Datagram_Type		datagramType;
};


/*
 * This list stores all the packet data received
 */
typedef std::vector<PacketData *> ReceivedPacketDataList;


/*
 * Lists to store allow IP list and deny IP list
 */
typedef std::vector<unsigned long> IPList;

/*
 * Class for handling packet display functionality
 */
class CSfView
{
public:
	virtual void HandleNewPacket(PacketData *pPacketData) = 0;
};

/*
 * All common functions prototype
 */
void RepresentBytesAsHexString(std::wstring &oStr, void *inData, int inStart, int inEnd);
void PrepareHexDump(std::wstring &oStr, void *inData, int inStart, int inEnd, int maxBytesInOneLine);
void GetSourceMacAddress(std::wstring &oStr, PacketData *inPacketData);
void GetDestMacAddress(std::wstring &oStr, PacketData *inPacketData);
void GetDatagramTypeString(std::wstring &oStr, PacketData *inPacketData);
void GetEthernetProtocolName(std::wstring &oStr, PacketData *inPacketData);
void GetSourceIP(std::wstring &oStr, PacketData *inPacketData);
void GetDestIP(std::wstring &oStr, PacketData *inPacketData);
bool IsIPV4Datagram(PacketData *inPacketData);
void ConvertBytesToIPString(std::wstring &oStr, unsigned char *inData);
void SetBitOn(int *pValue,int BitNumber);
void ApplyFirewallOnPacket(PacketData *inPacketData);
void SendTCPResetBack(PacketData *inPacketData);
u16 ip_checksum_calc(u16 len_in_octets, u8 *header_buff);
u16 tcp_checksum_calc(u16 len_tcp, u8 *src_addr, u8	*dest_addr, BOOL padding, u8 *buff);
unsigned long ConvertStrIPToNumber(std::string &inDenyIPStr);
int ActivateSniffer(RNIF_ADAPTER_INFO& inAdapterInfo);
int DeactivateSniffer();
int SetFilterDirected();
int SetFilterPromiscuous();
int ProcessPacket(PacketData *pPacketData);
bool ActivateFirewall(wchar_t *fileName);
void DeactivateFirewall();
void SfInitialize(CSfView *pView);


#endif